Attribute VB_Name = "modListView"
' Modulname : vbzListView.bas
' Version   : This is only a part of the module for demo, complete version
'             on www.vb-zentrum.de -> Downloads -> Source code available
' Date      : 28.02.2011
' Source    : http://www.vb-zentrum.de
' Author    : ralf schlegel

' Please note:
' needs Microsoft Windows Common Controls 5.0(SP2) - comctl32.ocx,
' because many functions, such as "lvwSetExtendedStyle" are not
' supported in version 6.0

Option Explicit

' Constants:
Public Const LVM_FIRST As Long = &H1000&
Public Const LVM_SETTEXTBKCOLOR As Long = (LVM_FIRST + 38)
Public Const LVM_SETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 54)
Public Const LVM_GETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 55)
Public Const LVM_SETBKIMAGE As Long = (LVM_FIRST + 68)
Public Const LVM_GETBKIMAGE As Long = (LVM_FIRST + 69)

' LVM_SET/GETBKIMAGE Flags
Private Const LVBKIF_SOURCE_NONE As Long = &H0
Private Const LVBKIF_SOURCE_HBITMAP As Long = &H1         ' WinXP or higher, only visible in compiled mode
Private Const LVBKIF_SOURCE_URL As Long = &H2
Private Const LVBKIF_SOURCE_MASK As Long = &H3
Private Const LVBKIF_STYLE_NORMAL As Long = &H0
Private Const LVBKIF_STYLE_TILE As Long = &H10
Private Const LVBKIF_TYPE_WATERMARK As Long = &H10000000  ' WinXP or higher, only visible in compiled mode
Private Const LVBKIF_FLAG_ALPHABLEND As Long = &H20000000 ' WinXP or higher, only visible in compiled mode

' ListView extended styles
Public Enum LVW_EX_STYLES
  LVS_EX_GRIDLINES = &H1&
  LVS_EX_SUBITEMIMAGES = &H2&
  LVS_EX_CHECKBOXES = &H4&
  LVS_EX_TRACKSELECT = &H8&
  LVS_EX_HEADERDRAGDROP = &H10&
  LVS_EX_FULLROWSELECT = &H20&
  LVS_EX_ONECLICKACTIVATE = &H40&
  LVS_EX_TWOCLICKACTIVATE = &H80&
  LVS_EX_FLATSB = &H100&
  LVS_EX_REGIONAL = &H200&
  LVS_EX_INFOTIP = &H400&
  LVS_EX_UNDERLINEHOT = &H800&
  LVS_EX_UNDERLINECOLD = &H1000&
  LVS_EX_MULTIWORKAREAS = &H2000&
  LVS_EX_LABELTIP = &H4000&
  LVS_EX_BORDERSELECT = &H8000&
  LVS_EX_DOUBLEBUFFER = &H10000         ' minimum OS: WinXP!
  LVS_EX_HIDELABELS = &H20000
  LVS_EX_SINGLEROW = &H40000
  LVS_EX_SNAPTOGRID = &H80000
  LVS_EX_SIMPLESELECT = &H100000
  LVS_EX_HEADERINALLVIEWS = &H2000000   ' minimum OS: Vista (header in all views)!
End Enum   ' LVW_EX_STYLES

Private Const HDS_BUTTONS As Long = &H2
Private Const WM_VSCROLL As Long = &H115

Private Const GWL_EXSTYLE As Long = -20
Private Const GWL_HINSTANCE As Long = -6
Private Const GWL_HWNDPARENT As Long = -8
Private Const GWL_ID As Long = -12
Private Const GWL_STYLE As Long = -16
Private Const GWL_USERDATA As Long = -21
Private Const GWL_WNDPROC As Long = -4
Private Const SWP_DRAWFRAME = &H20
Private Const SWP_NOMOVE = &H2
Private Const SWP_NOSIZE = &H1
Private Const SWP_NOZORDER = &H4
Private Const SWP_FLAGS = SWP_NOZORDER Or SWP_NOSIZE Or SWP_NOMOVE Or SWP_DRAWFRAME
Private Const WS_CLIPCHILDREN = &H2000000

Private Type LVBKIMAGE
  ulFlags As Long                       ' LVBKIF_*
  hBM As Long                           ' bitmap handle
  pszImage As String                    ' file name
  cchImageMax As Long                   ' size of max file name
  xOffsetPercent As Long                ' x offset
  yOffsetPercent As Long                ' y offset
End Type
Private Const CLR_DEFAULT = &HFF000000  ' default background color
Private Const CLR_NONE = &HFFFFFFFF     ' no background color (transparent)

' Declaration window and system functions:
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, _
        ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function SendMessageLong Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, _
        ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" ( _
        ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, _
        ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function EnableWindow Lib "user32" (ByVal hWnd As Long, ByVal fEnable As Long) As Long
Private Declare Function FindWindowEx Lib "user32" Alias "FindWindowExA" (ByVal hWnd1 As Long, _
        ByVal hWnd2 As Long, ByVal lpsz1 As String, ByVal lpsz2 As String) As Long

' ***  Private graphic functions (gdi32) for background and watermark effect:
Private Declare Function GetObjectAPI Lib "gdi32" Alias "GetObjectA" ( _
        ByVal hObject As Long, ByVal nCount As Long, ByRef lpObject As Any) As Long
Private Declare Function CreateDCAsNull Lib "gdi32" Alias "CreateDCA" (ByVal lpDriverName As String, _
        lpDeviceName As Any, lpOutput As Any, lpInitData As Any) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, _
        ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, _
        ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, _
        ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Private Type BITMAP
  bmType As Long
  bmWidth As Long
  bmHeight As Long
  bmWidthBytes As Long
  bmPlanes As Long
  bmBitsPixel As Integer
  bmBits As Long
End Type
' *** End graphic functions

' *** Begin XP themes and styles
Private Declare Function LoadLibrary Lib "kernel32" Alias "LoadLibraryA" (ByVal lpLibFileName As String) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function SetWindowTheme Lib "uxtheme" (ByVal hWnd As Long, ByVal pSubAppName As Long, _
        ByVal pSubIDList As Long) As Long
Private Enum puxTHEME
  uxUNKNOWN = &H0&
  uxNONE = &H1&
  uxOKAY = &H2&
End Enum

Private uxtheme As puxTHEME
' *** End XP themes

' ListView background image for Common Controls 5.0
Public Sub lvwSetBackgroundImage(ByVal lvHwnd As Long, Optional ByVal imageFileName As String = vbNullString, _
                                 Optional ByVal iPic As IPicture = Nothing, Optional ByVal tile As Boolean = True, _
                                 Optional ByVal xOffPercent As Long = 0, Optional ByVal yOffPercent As Long = 0)
  Dim bgi As LVBKIMAGE
  
  With bgi
    ' first clear up any other background and watermark:
    .ulFlags = LVBKIF_SOURCE_NONE
    SendMessage lvHwnd, LVM_SETBKIMAGE, 0, bgi
    .ulFlags = LVBKIF_TYPE_WATERMARK
    SendMessage lvHwnd, LVM_SETBKIMAGE, 0, bgi
    ' set new background picture
    If Len(imageFileName) Then
      .pszImage = imageFileName
      .cchImageMax = Len(.pszImage)
      .ulFlags = LVBKIF_SOURCE_URL
    Else
      .hBM = pBitmapHandleFromPicture(iPic)                   ' needs WinXP or higher!
      .ulFlags = LVBKIF_SOURCE_HBITMAP                        ' only visible in compiled exe!
    End If
    If tile Then .ulFlags = .ulFlags Or LVBKIF_STYLE_TILE
    .xOffsetPercent = xOffPercent
    .yOffsetPercent = yOffPercent
  End With
  SendMessage lvHwnd, LVM_SETBKIMAGE, 0, bgi                  ' set the background
  SendMessageLong lvHwnd, LVM_SETTEXTBKCOLOR, 0&, -1&         ' set text background transparent
End Sub

' create watermark in Listview Control Version 5.0
' needs Windows XP or newer, only visible in compiled code!
Public Sub lvwSetWatermark(ByVal lvHwnd As Long, ByVal iPic As IPicture)
  Dim bgi As LVBKIMAGE
  
  With bgi
    ' first clear up any background:
    .ulFlags = LVBKIF_SOURCE_NONE
    SendMessage lvHwnd, LVM_SETBKIMAGE, 0, bgi
    ' then set watermark image:
    .ulFlags = LVBKIF_TYPE_WATERMARK
    .hBM = pBitmapHandleFromPicture(iPic)
    SendMessage lvHwnd, LVM_SETBKIMAGE, 0, bgi
    SendMessageLong lvHwnd, LVM_SETTEXTBKCOLOR, 0&, -1&
  End With
End Sub

' ListView ExplorerStyle, only for Common Controls 5.0
' needs Windows XP or newer, only visible in compiled code!
Public Function lvwSetExplorerStyle(ByVal lvHwnd As Long) As Boolean
  Dim hModule As Long
  ' check if uxtheme.dll exist (only once a session for better performance!)
  If uxtheme = uxUNKNOWN Then                   ' function called first time
    hModule = LoadLibrary("uxtheme.dll")        ' try to load uxtheme.dll
    If hModule Then                             ' if exist, then
      uxtheme = uxOKAY                          ' keep success in memory
      FreeLibrary hModule                       ' and unload the library
    Else
      uxtheme = uxNONE                          ' keep info: no theme dll
    End If
  End If
  
  If uxtheme = uxOKAY Then                      ' if dll exist (see above)
    SetWindowTheme lvHwnd, StrPtr("explorer"), 0  ' set the explorer style
    lvwSetExplorerStyle = True                  ' and return "TRUE"
  End If
End Function

' ListView style extensions, only Common Controls 5.0
Public Sub lvwSetExtendedStyle(ByVal lvHwnd As Long, ByVal lStyle As LVW_EX_STYLES, ByVal flag As Boolean)
  Dim mask As Long
  
  mask = SendMessageLong(lvHwnd, LVM_GETEXTENDEDLISTVIEWSTYLE, 0&, 0&)
  If flag Then                  ' switch on style
    mask = mask Or lStyle
  Else                          ' switch off
    mask = mask And Not lStyle
  End If
  SendMessageLong lvHwnd, LVM_SETEXTENDEDLISTVIEWSTYLE, ByVal 0&, ByVal mask
  lStyle = GetWindowLong(lvHwnd, GWL_STYLE) Or WS_CLIPCHILDREN
  SetWindowLong lvHwnd, GWL_STYLE, lStyle
End Sub

' ######################################################################################################## '
' # PRIVATE (p) HELPER FUNCTIONS                                                                         # '
' ######################################################################################################## '

' create a copy of an IPicture object in memory and return its handle
Private Function pBitmapHandleFromPicture(ByVal iPic As IPicture) As Long
  Dim hDCDesktop As Long, hBMP As Long
  Dim hDC1 As Long, hBmpOld1 As Long
  Dim hDC2 As Long, hBmpOld2 As Long
  Dim tBMP As BITMAP
   
  If iPic Is Nothing Then Exit Function
  GetObjectAPI iPic.Handle, Len(tBMP), tBMP
  hDCDesktop = CreateDCAsNull("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
  If hDCDesktop Then
    hDC1 = CreateCompatibleDC(hDCDesktop)
    If hDC1 Then
      hBmpOld1 = SelectObject(hDC1, iPic.Handle)
      hDC2 = CreateCompatibleDC(hDCDesktop)
      If hDC2 Then
        hBMP = CreateCompatibleBitmap(hDCDesktop, tBMP.bmWidth, tBMP.bmHeight)
        If hBMP Then
          hBmpOld2 = SelectObject(hDC2, hBMP)
          BitBlt hDC2, 0, 0, tBMP.bmWidth, tBMP.bmHeight, hDC1, 0, 0, vbSrcCopy
          SelectObject hDC2, hBmpOld2
          pBitmapHandleFromPicture = hBMP
        End If  ' hBMP
        DeleteDC hDC2
      End If  ' hDC2
      SelectObject hDC1, hBmpOld1
      DeleteDC hDC1
    End If  ' hDC1
    DeleteDC hDCDesktop
  End If  ' hDCDesktop
End Function
